
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>

#include <math.h>
#ifndef M_PI
#define M_PI  3.14159265358979323846
#endif


#define STEP 10      /* number of degrees in the circle drawn as one straight line */

#define MAXVEC ((360 / STEP) + 4)        /* maximim number of Area calls (+2 for safety) */


#define WINW 200           /* size of drawing area */
#define WINH 200

#define CX (WINW >> 1)     /* center position */
#define CY (WINH >> 1)

#define RX (CX - 10)       /* radius */
#define RY (CY - 10)



void draw_pie_chart (struct RastPort *rp,UWORD cx,UWORD cy,UWORD rx,UWORD ry,UWORD step,UWORD n,ULONG *values,UBYTE *pens)

/* Parameters:

 rp      RastPort of drawing area. TmpRas and AreaInfo must already be initialised.
 cx/cy   coordinates of the center point
 rx/ry   radius in x and y direction
 step    number of degrees drawn as one line
 n       number of pieces
 values  array with the size of each piece
 pens    array with the color for each piece

*/

{
ULONG sum;         /* sum of all values (represents 360 degrees) */
ULONG a1,a2;       /* start and end angle of each piece in degrees */
ULONG value;       /* sum of values up to the end of the current piece */
ULONG i;           /* counter */
ULONG a;           /* angle in degrees */
FLOAT d;           /* angle in rad */
ULONG x,y;         /* coordinates of destination drawing point */


sum = 0;
for (i = 0; i < n; i++)      /* calculate sum */
	sum += values[i];

value = 0;
a2 = 0;                      /* start at zero angle and value */
for (i = 0; i < n; i++)
	{
	a1 = a2;                 /* current start angle is end angle of previous piece */
	value += values[i];      /* sum values up to the end of the current piece */
	a2 = value * 360 / sum;  /* calculate current end angle */
	SetAPen (rp,pens[i]);    /* set drawing pen */
	AreaMove (rp,cx,cy);     /* start area in the center point */

	for (a = a1; a < a2; a += step)
		{
		d = a * M_PI / 180.0;          /* calculate rad from degrees */
		x = fcos(d) * (FLOAT)rx;       /* calculate circle point */
		y = fsin(d) * (FLOAT)ry;
		AreaDraw (rp,x + cx,y + cy);   /* draw to circle point */
		}

	d = a2 * M_PI / 180.0;             /* last point is at end angle, even if step does not exactly hit it */
	x = fcos(d) * (FLOAT)rx;
	y = fsin(d) * (FLOAT)ry;
	AreaDraw (rp,x + cx,y + cy);

	AreaEnd(rp);                       /* complete area and fill it */
	}
}


int main (void)

{
struct Window *win;
struct MsgPort *port;
struct IntuiMessage *mess;
BOOL cont;
struct RastPort *rp;
struct TmpRas tmpras;
UBYTE *tmpbuf;
ULONG rassize;
struct AreaInfo areainfo;
UBYTE *areabuf;

static ULONG pie[] = {20,60,30,80};
static UBYTE pen[] = { 1, 2, 3, 4};

if (win = OpenWindowTags (NULL,
		WA_Left,10,WA_Top,20,
		WA_InnerWidth,WINW,
		WA_InnerHeight,WINH,
		WA_Title,"Press Esc to exit",
		WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_ACTIVATE|WFLG_GIMMEZEROZERO,
		WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_VANILLAKEY,
		TAG_END))
	{
	rp = win->RPort;

	rassize = RASSIZE(win->GZZWidth,win->GZZHeight);
	if (tmpbuf = AllocVec (rassize,MEMF_CHIP|MEMF_CLEAR))
		{
		InitTmpRas (&tmpras,tmpbuf,rassize);
		rp->TmpRas = &tmpras;
		}

	if (areabuf = AllocVec (5*MAXVEC,MEMF_CLEAR))
		{
		InitArea (&areainfo,areabuf,MAXVEC);
		rp->AreaInfo = &areainfo;
		}

	draw_pie_chart (rp,CX,CY,RX,RY,STEP,4,pie,pen);

	port = win->UserPort;
	cont = TRUE;
	while (cont)
		{
		WaitPort (port);
		while (mess = (struct IntuiMessage *) GetMsg (port))
			{
			switch (mess->Class)
				{
			case IDCMP_CLOSEWINDOW:
				cont = FALSE;
				break;
			case IDCMP_VANILLAKEY:
				if (mess->Code == 0x1b)
					cont = FALSE;
				}
			ReplyMsg ((struct Message *) mess);
			}
		}

	if (tmpbuf)
		{
		rp->TmpRas = NULL;
		FreeVec (tmpbuf);
		}

	if (areabuf)
		{
		rp->AreaInfo = NULL;
		FreeVec (areabuf);
		}

	CloseWindow (win);
	}
else
	return (20);

return (0);
}
